<?php

namespace Drupal\poster_integration\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\poster_integration\PosterConnection;
use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\poster_integration\Service\LoadHelper;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Form to batch load categories from POSTER API.
 */
class LoadCategoriesForm extends FormBase {

  /**
   * Batch Builder.
   *
   * @var \Drupal\Core\Batch\BatchBuilder
   */
  protected $batchBuilder;

  /**
   * The terms storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $termStorage;

  /**
   * Helper service.
   *
   * @var \Drupal\poster_integration\Service\LoadHelper
   */
  protected $helper;

  /**
   * BatchForm constructor.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, LoadHelper $helper) {
    $this->termStorage = $entity_type_manager->getStorage('taxonomy_term');
    $this->helper = $helper;
    $this->batchBuilder = new BatchBuilder();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('entity_type.manager'), $container->get('poster_integration.load_helper'));
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'load_categories_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['description'] = [
      '#type' => 'markup',
      '#markup' => '<p>' . $this->t('Press Load Categories button to get all categories from your Poster account to your Commerce') . '</p>',
    ];
    $vocabulary = $this->config('poster_integration.settings')->get('catvocabulary') ;
    if ($vocabulary) {
      $form['submit'] = [
        '#type' => 'submit',
        '#value' => 'Load Categories',
      ];
      $form['vocabulary'] = [
        '#type' => 'hidden',
        '#value' => $vocabulary,
      ];
    } else {
      $form['vocabulary'] = [
        '#type' => 'markup',
        '#markup' => '<h3>' . $this->t('Set vocabulary for categories import on settings page') . '</h3>',
      ];
    }

    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $vid = $this->config('poster_integration.settings')->get('catvocabulary');
    $connection = new PosterConnection();
    $result = $connection->makeRequest('menu.getCategories', 'get');
    $decoded_result = json_decode($result);
    $categories = [];
    if (empty($decoded_result->error)) {
      $categories = $decoded_result->response;
    }
    $this->batchBuilder
      ->setTitle($this->t('Processing'))
      ->setInitMessage($this->t('Initializing.'))
      ->setProgressMessage($this->t('Completed @current of @total.'))
      ->setErrorMessage($this->t('An error has occurred.'));

    $this->batchBuilder->setFile(drupal_get_path('module', 'poster_integration') . '/src/Form/LoadCategoriesForm.php');
    $this->batchBuilder->addOperation([$this, 'processItems'], [$categories, $vid]);
    $this->batchBuilder->setFinishCallback([$this, 'finished']);

    batch_set($this->batchBuilder->toArray());
  }

  /**
   * Processor for batch operations.
   */
  public function processItems($items, $vid, array &$context) {
    // Elements per operation.
    $limit = 10;

    // Set default progress values.
    if (empty($context['sandbox']['progress'])) {
      $context['sandbox']['progress'] = 0;
      $context['sandbox']['max'] = count($items);
    }

    // Save items to array which will be changed during processing.
    if (empty($context['sandbox']['items'])) {
      $context['sandbox']['items'] = $items;
    }

    $counter = 0;
    if (!empty($context['sandbox']['items'])) {
      // Remove already processed items.
      if ($context['sandbox']['progress'] != 0) {
        array_splice($context['sandbox']['items'], 0, $limit);
      }

      foreach ($context['sandbox']['items'] as $item) {
        if ($counter != $limit) {
          $this->processItem($item, $vid);

          $counter++;
          $context['sandbox']['progress']++;

          $context['message'] = $this->t('Now processing category :progress of :count', [
            ':progress' => $context['sandbox']['progress'],
            ':count' => $context['sandbox']['max'],
          ]);

          // Increment total processed item values. Will be used in finished
          // callback.
          $context['results']['processed'] = $context['sandbox']['progress'];
        }
      }
    }

    // If not finished all tasks, we count percentage of process. 1 = 100%.
    if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
      $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    }
  }

  /**
   * Process single item.
   *
   * @param int|string $nid
   *   An id of Node.
   */
  public function processItem($category, $vid) {
    $taxonomy_id = $this->helper->getTaxonomyByPosterID($category->category_id);
/*     $result = \Drupal::entityQuery('taxonomy_term')
       ->condition('vid', $vid)
       ->execute();
     entity_delete_multiple('taxonomy_term', $result);*/
    $this->helper->saveTaxonomy($category, $vid, $taxonomy_id);
  }

  /**
   * Finished callback for batch.
   */
  public function finished($success, $results, $operations) {
    $message = $this->t('Number of categories affected by batch: @count', [
      '@count' => $results['processed'],
    ]);

    $this->messenger()
      ->addStatus($message);
    return new RedirectResponse(Url::fromRoute('poster_integration.get_products', [], ['absolute' => TRUE])->toString());
  }

}
