<?php

namespace Drupal\poster_integration\Form;

use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\poster_integration\PosterConnection;
use Drupal\poster_integration\Service\LoadHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Url;

/**
 * Form to batch load products from POSTER API.
 */
class LoadProductsForm extends FormBase {

  /**
   * Batch Builder.
   *
   * @var \Drupal\Core\Batch\BatchBuilder
   */
  protected $batchBuilder;

  /**
   * Helper service.
   *
   * @var \Drupal\poster_integration\Service\LoadHelper
   */
  protected $helper;

  /**
   * BatchForm constructor.
   */
  public function __construct(LoadHelper $helper) {
    $this->helper = $helper;
    $this->batchBuilder = new BatchBuilder();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static($container->get('poster_integration.load_helper'));
  }
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'load_products_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['description'] = [
      '#type' => 'markup',
      '#markup' => '<p>' . $this->t('Press Load Products button to get all products from your Poster account to your Commerce') . '</p>',
    ];

    $prodtype = $this->config('poster_integration.settings')->get('prodtype');
    if ($prodtype) {
      $form['submit'] = [
        '#type' => 'submit',
        '#value' => 'Load Products',
      ];
      $form['prodtype'] = [
        '#type' => 'hidden',
        '#value' => $prodtype,
      ];
    }
    else {
      $form['prodtype'] = [
        '#type' => 'markup',
        '#markup' => '<h3>' . $this->t('Set Product type for products import on settings page') . '</h3>',
      ];
    }
    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $prodtype = $this->config('poster_integration.settings')->get('prodtype');
    $connection = new PosterConnection();
    $result = $connection->makeRequest('menu.getProducts', 'get');
    $decoded_result = json_decode($result);
    $products = [];
    if (empty($decoded_result->error)) {
      $products = $decoded_result->response;
    }
    $this->batchBuilder
      ->setTitle($this->t('Processing'))
      ->setInitMessage($this->t('Initializing.'))
      ->setProgressMessage($this->t('Completed @current of @total.'))
      ->setErrorMessage($this->t('An error has occurred.'));

    $this->batchBuilder->setFile(drupal_get_path('module', 'poster_integration') . '/src/Form/LoadProductsForm.php');
    $this->batchBuilder->addOperation([$this, 'processItems'], [$products, $prodtype]);
    $this->batchBuilder->setFinishCallback([$this, 'finished']);

    batch_set($this->batchBuilder->toArray());
  }

  /**
   * Processor for batch operations.
   */
  public function processItems($items, $prodtype, array &$context) {
    // Elements per operation.
    $limit = 10;

    // Set default progress values.
    if (empty($context['sandbox']['progress'])) {
      $context['sandbox']['progress'] = 0;
      $context['sandbox']['max'] = count($items);
    }

    // Save items to array which will be changed during processing.
    if (empty($context['sandbox']['items'])) {
      $context['sandbox']['items'] = $items;
    }

    $counter = 0;
    if (!empty($context['sandbox']['items'])) {
      // Remove already processed items.
      if ($context['sandbox']['progress'] != 0) {
        array_splice($context['sandbox']['items'], 0, $limit);
      }

      foreach ($context['sandbox']['items'] as $item) {
        if ($counter != $limit) {
          $this->processItem($item, $prodtype);

          $counter++;
          $context['sandbox']['progress']++;

          $context['message'] = $this->t('Now processing product :progress of :count', [
            ':progress' => $context['sandbox']['progress'],
            ':count' => $context['sandbox']['max'],
          ]);

          // Increment total processed item values. Will be used in finished
          // callback.
          $context['results']['processed'] = $context['sandbox']['progress'];
        }
      }
    }

    // If not finished all tasks, we count percentage of process. 1 = 100%.
    if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
      $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    }
  }

  /**
   * Process single item.
   *
   * @param array $product
   *   Array of products
   * @param string $vid
   *   Product type
   */
  public function processItem($product, $prodtype) {
    $this->helper->saveProduct($product, $prodtype);
  }

  /**
   * Finished callback for batch.
   */
  public function finished($success, $results, $operations) {
    $message = $this->t('Number of products affected by batch: @count', [
      '@count' => $results['processed'],
    ]);

    $this->messenger()
      ->addStatus($message);
    return new RedirectResponse(Url::fromRoute('poster_integration.get_products', [], ['absolute' => TRUE])->toString());
  }


}
